#if !defined DBOARD_H
#define DBOARD_H

#include <X11/Xlib.h>
#include "output_color.h"

enum cursorShape {
  CURSOR_WAIT,
  CURSOR_MAX
};

struct dBoardImage {
  XImage *ximage;
  int xoff;
  int yoff;
  int width;
  int height;
};
typedef struct dBoardImage DBoardImage;
typedef struct dBoard DBoard;
typedef int Handler(DBoard *board, XEvent *event, int *pdone, void *data);
typedef int Processor(DBoard *board, int *pdone, void *data);

DBoard *dBoard_open(const char *progname, int *argc, char **argv);
void dBoard_get_attributes(DBoard *board, int *depth, int *width, int *height, unsigned short *bg_r, unsigned short *bg_g, unsigned short *bg_b, int *bg_setted);
void dBoard_set_name(DBoard *board, const char *name);
void dBoard_show(DBoard *board);
int dBoard_close(DBoard *board);
int dBoard_main(DBoard *board, void *data);
int dBoard_flush(DBoard *board);
void dBoard_reset(DBoard *board, int width, int height);
void dBoard_copy(DBoard *board);
void dBoard_define_cursor(DBoard *board, enum cursorShape shape);
void dBoard_undefine_cursor(DBoard *board);
OutputColor *dBoard_get_colors(DBoard *board, int bit_depth);
unsigned long dBoard_get_pixel_rgb16(DBoard *board, unsigned char *p);

void dBoard_fill_rectangle(DBoard *board, int x, int y, unsigned int width, unsigned int height, unsigned long pixel);
DBoardImage *dBoard_get_image(DBoard *board, int xoff, int yoff, int width, int height);
int dBoard_put_image(DBoard *board, DBoardImage *bimage);

Atom dBoard_delete_window(DBoard *board);
int dBoard_setHandler(DBoard *board, Handler *handler, long event_mask);
void dBoard_setProcessor(DBoard *board, Processor *processor);

#endif /* !DBOARD_H */
