#if !defined IMAGESET_H
#define IMAGESET_H

#include "cmsprofile.h"

#if !defined TRUE
#define TRUE !0
#define FALSE 0
#endif /* !TRUE */

enum imageType {
  IMAGE_TYPE_UNKNOWN,
  IMAGE_TYPE_INDEXED_GRAY_8,  /* Indexed GRAY_8 colors */
  IMAGE_TYPE_INDEXED_GRAYA_8, /* Indexed GRAYA_8 colors */
  IMAGE_TYPE_INDEXED_RGB_8,   /* Indexed RGB_8 colors */
  IMAGE_TYPE_INDEXED_RGBA_8,  /* Indexed RGBA_8 colors */
  IMAGE_TYPE_GRAY_8,    /* GRAY 8 bits per component */
  IMAGE_TYPE_GRAY_16,   /* GRAY 16 bits per component */
  IMAGE_TYPE_GRAYA_8,   /* GRAYA 8 bits per component */
  IMAGE_TYPE_GRAYA_16,  /* GRAYA 16 bits per component */
  IMAGE_TYPE_RGB_8,     /* RGB 8 bits per component */
  IMAGE_TYPE_RGB_16,    /* RGB 16 bits per component */
  IMAGE_TYPE_RGBA_8,    /* RGBA 8 bits per component */
  IMAGE_TYPE_RGBA_16,   /* RGBA 16 bits per component */
  IMAGE_TYPE_CMYK_8,    /* CMYK 8 bits per component */
  IMAGE_TYPE_CMYK_8_REV /* CMYK 8 bits per component */
};
typedef enum imageType ImageType;

enum disposeOp {
  DISPOSE_OP_NONE,       /* no disposal is done */
  DISPOSE_OP_BACKGROUND, /* to be cleared to fully transparent black */
  DISPOSE_OP_PREVIOUS    /* to be reverted to the previous contents */
};

enum blendOp {
  BLEND_OP_NONE,   /* no blend is done */
  BLEND_OP_SOURCE, /* overwrite the buffer */
  BLEND_OP_OVER    /* to be composited onto the buffer */
};

typedef struct image Image;
typedef struct imageSet ImageSet;


ImageSet *imageSet_new(void);
void imageSet_free(ImageSet *images);
void imageSet_init(ImageSet *images);
void imageSet_add_image(ImageSet *images, Image *image);
Image *imageSet_get_first(ImageSet *images);
int imageSet_get_num_frames(ImageSet *images);
int imageSet_get_background(ImageSet *images, unsigned char *p, ImageType *ptype);
void imageSet_set_background(ImageSet *images, const unsigned char *p, ImageType type);
void imageSet_get_size(ImageSet *images, int *width, int *height);
void imageSet_set_size(ImageSet *images, int width, int height);
int imageSet_get_num_plays(ImageSet *images);
void imageSet_set_num_plays(ImageSet *images, int num);
int imageSet_get_num_indexes(ImageSet *images);
void imageSet_set_num_indexes(ImageSet *images, int num);
void imageSet_get_index(ImageSet *images, int index, unsigned char *data);
void imageSet_set_index(ImageSet *images, int index, const unsigned char *data);
CmsProfile *imageSet_get_profile(ImageSet *images);
void imageSet_set_profile(ImageSet *images, CmsProfile *profile);

Image *image_new(void);
void image_free(Image *image);
unsigned char *image_get_data(Image *image);
void image_set_data(Image *image, unsigned char *data);
void image_get_size(Image *image, int *width, int *height);
void image_set_size(Image *image, int width, int height);
void image_get_offset(Image *image, int *xoff, int *yoff);
void image_set_offset(Image *image, int xoff, int yoff);
int image_get_num_indexes(Image *image);
void image_set_num_indexes(Image *image, int num);
void image_get_index(Image *image, int index, unsigned char *data);
void image_set_index(Image *image, int index, const unsigned char *data);
void image_set_comment(Image *image, const char *comment);
int image_get_delay(Image *image);
void image_set_delay(Image *image, int delay);
int image_get_dispose_op(Image *image);
void image_set_dispose_op(Image *image, int dispose_op);
int image_get_blend_op(Image *image);
void image_set_blend_op(Image *image, int blend_op);
ImageType image_get_type(Image *image);
void image_set_type(Image *image, ImageType type);
Image *image_next(Image *image);
int imageType_bytes_per_pixel(ImageType type);

#endif /* !IMAGESET_H */
