#include "common.h"
#include "loader.h"
#include "module.h"
#include "imageset.h" /* imageSet_init() */

#if defined ENABLE_GIF
extern Module module_GIF;
#endif /* ENABLE_GIF */
#if defined ENABLE_JPEG
extern Module module_JPEG;
#endif /* ENABLE_JPEG */
#if defined ENABLE_PNG
extern Module module_PNG;
#endif /* ENABLE_PNG */
#if defined ENABLE_TIFF
extern Module module_TIFF;
#endif /* ENABLE_TIFF */
extern Module module_xbm;
extern Module module_null;


int load_images(Entry *entry, ImageSet *images, const char *format)
{
  const Module modules[] = {
#if defined ENABLE_GIF
    module_GIF,
#endif /* ENABLE_GIF */
#if defined ENABLE_JPEG
    module_JPEG,
#endif /* ENABLE_JPEG */
#if defined ENABLE_PNG
    module_PNG,
#endif /* ENABLE_PNG */
#if defined ENABLE_TIFF
    module_TIFF,
#endif /* ENABLE_TIFF */
    module_xbm,
    module_null
  };
  Module *module;

  if (format == NULL) {
    FILE *fp;
    char buf[16];
    size_t num;
    fp = fopen(entry->filename, "rb");
    if (fp == NULL) {
      return FALSE;
    }
    num = fread(buf, sizeof(char), 16, fp);
    if (num < 16) {
      if (ferror(fp)) {
        fclose(fp);
        return FALSE;
      }
    }
    fclose(fp);
    module = modules;
    while (module->magic != NULL) {
      if (module->magic(buf, sizeof(char)*num)) {
        break;
      }
      module++;
    }
    if (module->magic == NULL) {
      fprintf(stderr, "Unknown file type '%s'\n", entry->filename);
    }
  } else {
    module = modules;
    while (module->name != NULL) {
      if (strcmp(format, module->name) == 0) {
        break;
      }
      module++;
    }
    if (module->name == NULL) {
      fprintf(stderr, "Unknown format type '%s'\n", format);
    }
  }
  imageSet_init(images);
  if (!(*(module->reader))(entry, images)) {
    fprintf(stderr, "Can not read file '%s'\n", entry->filename);
    return FALSE;
  }
  return TRUE;
}
