#include <X11/Xlib.h> /* XColor */
#include "common.h"
#include "output_color.h"


struct outputColor {
  XColor *xcolors;
  unsigned int num;
  int bit_depth;
};


OutputColor *outputColor_alloc(int bit_depth)
{
  OutputColor *color;
  color = XMALLOC(OutputColor, 1);
  if (color != NULL) {
    XColor *xcolors;
    unsigned int num;
    num = 0x1 << bit_depth;
    xcolors = XMALLOC(XColor, num*3);
    if (xcolors == NULL) {
      XFREE(color);
    }
    color->xcolors = xcolors;
    color->num = num;
    color->bit_depth = bit_depth;
  }
  return color;
}

void outputColor_free(OutputColor *color)
{
  if (color != NULL) {
    XFREE(color->xcolors);
    XFREE(color);
  }
}

unsigned long outputColor_get_pixel(OutputColor *color, unsigned short r, unsigned short g, unsigned short b)
{
  unsigned long pixel = 0xff;
  unsigned int num;
  num = color->num;
  if (((unsigned int)r < num) && ((unsigned int)g < num) && ((unsigned int)b < num)) {
    XColor *xcolors = color->xcolors;
    pixel  = xcolors[r].pixel;
    pixel |= xcolors[num+g].pixel;
    pixel |= xcolors[num*2+b].pixel;
  }
  return pixel;
}

XColor *outputColor_get_xcolor(OutputColor *color, unsigned int channel, unsigned short index)
{
  XColor *xcolor;
  if (channel < 3) {
    XColor *xcolors = color->xcolors;
    xcolor = &xcolors[color->num*channel+index];
  }
  return xcolor;
}
